package cn.kdan.compdf.filter;

import cn.kdan.compdf.config.JwtConfig;
import cn.kdan.compdf.dto.LoginUserDTO;
import cn.kdan.compdf.enums.AuthEnum;
import cn.kdan.compdf.enums.ResponseEnum;
import cn.kdan.compdf.exception.BusinessException;
import com.alibaba.fastjson.JSON;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cloud.gateway.filter.GatewayFilterChain;
import org.springframework.cloud.gateway.filter.GlobalFilter;
import org.springframework.core.Ordered;
import org.springframework.core.io.buffer.DataBuffer;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.server.reactive.ServerHttpRequest;
import org.springframework.http.server.reactive.ServerHttpResponse;
import org.springframework.stereotype.Component;
import org.springframework.web.server.ServerWebExchange;
import reactor.core.publisher.Mono;

import java.nio.charset.StandardCharsets;
import java.util.function.Consumer;

/**
 * 全局登录鉴权过滤器
 * @author Bob 2022-10-12
 **/
@Component
public class AuthGlobalFilter implements GlobalFilter, Ordered {

    @Autowired
    private StringRedisTemplate redisTemplate;

    @Override
    public Mono<Void> filter(ServerWebExchange exchange, GatewayFilterChain chain) {
        // 1、判断放行请求地址(登录接口)
        String requestPath = exchange.getRequest().getPath().toString();
        // 登录
        if (requestPath.equals("/user-api/v1/user/login")) {
            return chain.filter(exchange);
        }
//        // 修改密码
//        if (requestPath.equals("/user-api/v1/user/changePassword")) {
//            return chain.filter(exchange);
//        }

        // 2、获取请求头jwt
        String authorization = exchange.getRequest().getHeaders().getFirst(AuthEnum.AUTHORIZATION);
        // 没有jwt
        if (StringUtils.isEmpty(authorization)) {
            ServerHttpResponse response = exchange.getResponse();
            response.setStatusCode(HttpStatus.OK);
            // 改变响应的类型
            response.getHeaders().setContentType(MediaType.APPLICATION_JSON);
            String fastResult = JSON.toJSONString(new BusinessException(ResponseEnum.NO_LOGIN));
            DataBuffer dataBuffer = response.bufferFactory().allocateBuffer().write(fastResult.getBytes(StandardCharsets.UTF_8));
            return response.writeWith(Mono.just(dataBuffer));
        }
        authorization = authorization.replaceFirst("Bearer ","");

        // 3、解析
        try {
            LoginUserDTO loginUserDTO = JwtConfig.checkJwt(authorization);
            // redie验证token是否过期
            Boolean flag = redisTemplate.hasKey(AuthEnum.USER_TOKEN_REDIS + authorization);
            if (null == flag || !flag) {
                ServerHttpResponse response = exchange.getResponse();
                response.setStatusCode(HttpStatus.OK);
                // 改变响应的类型
                response.getHeaders().setContentType(MediaType.APPLICATION_JSON);
                String fastResult = JSON.toJSONString(new BusinessException(ResponseEnum.TOKEN_TIMEOUT));
                DataBuffer dataBuffer = response.bufferFactory().allocateBuffer().write(fastResult.getBytes(StandardCharsets.UTF_8));
                return response.writeWith(Mono.just(dataBuffer));
            }

            Consumer<HttpHeaders> httpHeaders = httpHeader -> {
                httpHeader.set(AuthEnum.USER_ID_HEADER, loginUserDTO.getId().toString());
                httpHeader.set(AuthEnum.USER_TENANT_ID_HEADER, loginUserDTO.getTenantId().toString());
                httpHeader.set(AuthEnum.USER_NAME_HEADER, loginUserDTO.getUsername());
//                httpHeader.set(AuthEnum.AUTHORIZATION, authorization);
            };
            ServerHttpRequest serverHttpRequest = exchange.getRequest().mutate().headers(httpHeaders).build();
            exchange.mutate().request(serverHttpRequest).build();
            return chain.filter(exchange);
        } catch (Exception e) {
            ServerHttpResponse response = exchange.getResponse();
            response.setStatusCode(HttpStatus.OK);
            // 改变响应的类型
            response.getHeaders().setContentType(MediaType.APPLICATION_JSON);
            String fastResult = JSON.toJSONString(new BusinessException(ResponseEnum.TOKEN_ERROR));
            DataBuffer dataBuffer = response.bufferFactory().allocateBuffer().write(fastResult.getBytes(StandardCharsets.UTF_8));
            return response.writeWith(Mono.just(dataBuffer));
        }
    }

    @Override
    public int getOrder() {
        return 0;
    }
}
